@@ -15,7 +15,7 @@ from junit_xml import TestSuite, TestCase
 from pony.orm import Database, Required, db_session, Set, Optional, select, \
     PrimaryKey, RowNotFound, ERDiagramError, OperationalError
 
-from mutmut import BAD_TIMEOUT, OK_SUSPICIOUS, BAD_SURVIVED, UNTESTED, \
+from mutmut import MUTANT_STATUSES, BAD_TIMEOUT, OK_SUSPICIOUS, BAD_SURVIVED, UNTESTED, \
     OK_KILLED, RelativeMutationID, Context, mutate
 
 db = Database()
@@ -149,7 +149,7 @@ def ranges(numbers):
 
 @init_db
 @db_session
-def print_result_cache(show_diffs=False, dict_synonyms=None, print_only_filename=None, only_this_file=None):
+def print_result_cache(show_diffs=False, dict_synonyms=None, only_this_file=None):
     print('To apply a mutant on disk:')
     print('    mutmut apply <id>')
     print('')
@@ -163,9 +163,6 @@ def print_result_cache(show_diffs=False, dict_synonyms=None, print_only_filename
             print('')
             print("{} ({})".format(title, len(mutant_list)))
             for filename, mutants in groupby(mutant_list, key=lambda x: x.line.sourcefile.filename):
-                if print_only_filename is not None and print_only_filename != filename:
-                    continue
-
                 if only_this_file and filename != only_this_file:
                     continue
 
@@ -189,6 +186,14 @@ def print_result_cache(show_diffs=False, dict_synonyms=None, print_only_filename
     print_stuff('Untested/skipped', select(x for x in Mutant if x.status == UNTESTED))
 
 
+@init_db
+@db_session
+def print_result_ids_cache(desired_status):
+    status = MUTANT_STATUSES[desired_status]
+    mutant_query = select(x for x in Mutant if x.status == status)
+    print(" ".join(str(mutant.id) for mutant in mutant_query))
+
+
 def get_unified_diff(argument, dict_synonyms, update_cache=True, source=None):
     filename, mutation_id = filename_and_mutation_id_from_pk(argument)
     if source is None:
@@ -222,14 +227,18 @@ def _get_unified_diff(source, filename, mutation_id, dict_synonyms, update_cache
     return output
 
 
+def print_result_cache_junitxml(dict_synonyms, suspicious_policy, untested_policy):
+    print(create_junitxml_report(dict_synonyms, suspicious_policy, untested_policy))
+
+
 @init_db
 @db_session
-def print_result_cache_junitxml(dict_synonyms, suspicious_policy, untested_policy):
+def create_junitxml_report(dict_synonyms, suspicious_policy, untested_policy):
     test_cases = []
     mutant_list = list(select(x for x in Mutant))
     for filename, mutants in groupby(mutant_list, key=lambda x: x.line.sourcefile.filename):
         for mutant in mutants:
-            tc = TestCase("Mutant #{}".format(mutant.id), file=filename, line=mutant.line.line_number, stdout=mutant.line.line)
+            tc = TestCase("Mutant #{}".format(mutant.id), file=filename, line=mutant.line.line_number + 1, stdout=mutant.line.line)
             if mutant.status == BAD_SURVIVED:
                 tc.add_failure_info(message=mutant.status, output=get_unified_diff(mutant.id, dict_synonyms))
             if mutant.status == BAD_TIMEOUT:
@@ -246,7 +255,7 @@ def print_result_cache_junitxml(dict_synonyms, suspicious_policy, untested_polic
             test_cases.append(tc)
 
     ts = TestSuite("mutmut", test_cases)
-    print(TestSuite.to_xml_string([ts]))
+    return TestSuite.to_xml_string([ts])
 
 
 @init_db
